"use strict";
var Bluebird = require('bluebird');
function createPool() {
    var descriptors = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        descriptors[_i - 0] = arguments[_i];
    }
    var pool = {
        models: descriptors.reduce(function (m, d) {
            m[d.name] = d;
            return m;
        }, {}),
    };
    pool.resolve = function resolve(query, context) {
        if (!query || !query.model) {
            return Bluebird.reject(new Error('query or query.model was not specified.'));
        }
        var model = pool.models[query.model];
        if (!model) {
            return Bluebird.reject(new Error(query.model + " is not a valid model"));
        }
        if (!model.resolver) {
            return Bluebird.reject(new Error(query.model + " has no resolver"));
        }
        return model.resolver(query, context);
    };
    pool.sync = function sync(transaction, item, rip, context) {
        if (!item.model) {
            return Bluebird.reject(new Error('item.model was not specified.'));
        }
        var model = pool.models[item.model];
        if (!model) {
            return Bluebird.reject(new Error(item.model + " is not a valid model"));
        }
        if (!model.synchronizer) {
            return Bluebird.reject(new Error(item.model + " has no synchronizer"));
        }
        return model.synchronizer(transaction, item, rip, context);
    };
    descriptors.forEach(function (d) { return d.initialize(pool); });
    return pool;
}
exports.createPool = createPool;
